@@ -67,14 +67,9 @@ TURN_VERBOSE_NONE,0,0,0,0,0,0,0,
 DEFAULT_STUN_PORT,DEFAULT_STUN_TLS_PORT,0,0,1,
 0,0,0,0,
 "",
-#if !defined(TURN_NO_HIREDIS)
 "",0,
-#endif
 {
-  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0,0,NULL,NULL,NULL
-#if !defined(TURN_NO_HIREDIS)
-,NULL
-#endif
+  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0,0,NULL,NULL,NULL,NULL
 },
 {NULL, NULL, 0},{NULL, NULL, 0},
 NEV_UNKNOWN, 
@@ -484,6 +479,8 @@ static char Usage[] = "Usage: turnserver [options]\n"
 "						a log file. With this option everything will be going to the log file only\n"
 "						(unless the log file itself is stdout).\n"
 " --syslog					Output all log information into the system log (syslog), do not use the file output.\n"
+" --simple-log					This flag means that no log file rollover will be used, and the log file\n"
+"						name will be constructed as-is, without PID and date appendage.\n"
 " --stale-nonce					Use extra security with nonce value having limited lifetime (600 secs).\n"
 " -S, --stun-only				Option to set standalone STUN operation only, all TURN requests will be ignored.\n"
 "     --no-stun					Option to suppress STUN functionality, only TURN requests will be processed.\n"
@@ -493,7 +490,7 @@ static char Usage[] = "Usage: turnserver [options]\n"
 " --tls-alternate-server	<ip:port>		Set the TURN server to redirect the allocate requests (DTLS and TLS services).\n"
 "						Multiple alternate-server options can be set for load balancing purposes.\n"
 "						See the docs for more information.\n"
-" -C, --rest-api-separator	<SYMBOL>	This is the username/timestamp separator symbol (character) in TURN REST API.\n"
+" -C, --rest-api-separator	<SYMBOL>	This is the timestamp/username separator symbol (character) in TURN REST API.\n"
 "						The default value is ':'.\n"
 "     --max-allocate-timeout=<seconds>		Max time, in seconds, allowed for full allocation establishment. Default is 60.\n"
 "     --allowed-peer-ip=<ip[-ip]> 		Specifies an ip or range of ips that are explicitly allowed to connect to the \n"
@@ -547,7 +544,7 @@ static char AdminUsage[] = "Usage: turnadmin [command] [options]\n"
 	"	-D, --delete-st			delete a short-term mechanism user\n"
 	"	-l, --list			list all long-term mechanism users\n"
 	"	-L, --list-st			list all short-term mechanism users\n"
-#if !defined(TURN_NO_PQ) || !defined(TURN_NO_MYSQL)
+#if !defined(TURN_NO_PQ) || !defined(TURN_NO_MYSQL) || !defined(TURN_NO_HIREDIS)
 	"	-s, --set-secret=<value>	Add shared secret for TURN RESP API\n"
 	"	-S, --show-secret		Show stored shared secrets for TURN REST API\n"
 	"	-X, --delete-secret=<value>	Delete a shared secret\n"
@@ -597,6 +594,7 @@ enum EXTRA_OPTS {
 	AUTH_SECRET_TS_EXP, /* deprecated */
 	NO_STDOUT_LOG_OPT,
 	SYSLOG_OPT,
+	SIMPLE_LOG_OPT,
 	AUX_SERVER_OPT,
 	UDP_SELF_BALANCE_OPT,
 	ALTERNATE_SERVER_OPT,
@@ -633,7 +631,22 @@ enum EXTRA_OPTS {
 	NO_TLSV1_2_OPT
 };
 
-static struct option long_options[] = {
+struct myoption {
+        const char *name;     /* name of long option */
+        int has_arg;    /* whether option takes an argument */
+        int *flag;      /* if not NULL, set *flag to val when option found */
+        int val;        /* if flag is not NULL, value to set *flag to. */
+                        /* if flag is NULL, return value */
+};
+
+struct uoptions {
+	union {
+		const struct myoption *m;
+		const struct option *o;
+	} u;
+};
+
+static const struct myoption long_options[] = {
 				{ "listening-device", required_argument, NULL, 'd' },
 				{ "listening-port", required_argument, NULL, 'p' },
 				{ "tls-listening-port", required_argument, NULL, TLS_PORT_OPT },
@@ -688,6 +701,7 @@ static struct option long_options[] = {
 				{ "log-file", required_argument, NULL, 'l' },
 				{ "no-stdout-log", optional_argument, NULL, NO_STDOUT_LOG_OPT },
 				{ "syslog", optional_argument, NULL, SYSLOG_OPT },
+				{ "simple-log", optional_argument, NULL, SIMPLE_LOG_OPT },
 				{ "aux-server", required_argument, NULL, AUX_SERVER_OPT },
 				{ "udp-self-balance", optional_argument, NULL, UDP_SELF_BALANCE_OPT },
 				{ "alternate-server", required_argument, NULL, ALTERNATE_SERVER_OPT },
@@ -725,13 +739,13 @@ static struct option long_options[] = {
 				{ NULL, no_argument, NULL, 0 }
 };
 
-static struct option admin_long_options[] = {
+static const struct myoption admin_long_options[] = {
 				{ "key", no_argument, NULL, 'k' },
 				{ "add", no_argument, NULL, 'a' },
 				{ "delete", no_argument, NULL, 'd' },
 				{ "list", no_argument, NULL, 'l' },
 				{ "list-st", no_argument, NULL, 'L' },
-#if !defined(TURN_NO_PQ) || !defined(TURN_NO_MYSQL)
+#if !defined(TURN_NO_PQ) || !defined(TURN_NO_MYSQL) || !defined(TURN_NO_HIREDIS)
 				{ "set-secret", required_argument, NULL, 's' },
 				{ "show-secret", no_argument, NULL, 'S' },
 				{ "delete-secret", required_argument, NULL, 'X' },
@@ -1137,6 +1151,7 @@ static void set_option(int c, char *value)
 	case 'l':
 	case NO_STDOUT_LOG_OPT:
 	case SYSLOG_OPT:
+	case SIMPLE_LOG_OPT:
 	case 'c':
 	case 'n':
 	case 'h':
@@ -1259,6 +1274,8 @@ static void read_config_file(int argc, char **argv, int pass)
 						set_no_stdout_log(get_bool_value(value));
 					} else if((pass==0) && (c==SYSLOG_OPT)) {
 						set_log_to_syslog(get_bool_value(value));
+					} else if((pass==0) && (c==SIMPLE_LOG_OPT)) {
+						set_simple_log(get_bool_value(value));
 					} else if((pass == 0) && (c != 'u')) {
 					  set_option(c, value);
 					} else if((pass > 0) && (c == 'u')) {
@@ -1287,7 +1304,10 @@ static int adminmain(int argc, char **argv)
 	u08bits pwd[STUN_MAX_PWD_SIZE+1]="";
 	u08bits secret[AUTH_SECRET_SIZE+1]="";
 
-	while (((c = getopt_long(argc, argv, ADMIN_OPTIONS, admin_long_options, NULL)) != -1)) {
+	struct uoptions uo;
+	uo.u.m = admin_long_options;
+
+	while (((c = getopt_long(argc, argv, ADMIN_OPTIONS, uo.u.o, NULL)) != -1)) {
 		switch (c){
 		case 'k':
 			ct = TA_PRINT_KEY;
@@ -1313,7 +1333,7 @@ static int adminmain(int argc, char **argv)
 			ct = TA_LIST_USERS;
 			is_st = 1;
 			break;
-#if !defined(TURN_NO_PQ) || !defined(TURN_NO_MYSQL)
+#if !defined(TURN_NO_PQ) || !defined(TURN_NO_MYSQL) || !defined(TURN_NO_HIREDIS)
 		case 's':
 			ct = TA_SET_SECRET;
 			STRCPY(secret,optarg);
@@ -1466,9 +1486,9 @@ static void print_features(unsigned long mfn)
 #endif
 
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
-	TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "OpenSSL version: fresh enough\n");
+	TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "OpenSSL compile-time version 0x%llx: fresh enough\n",(unsigned long long)OPENSSL_VERSION_NUMBER);
 #else
-	TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "OpenSSL version: antique\n");
+	TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "OpenSSL compile-time version 0x%llx version: antique\n",(unsigned long long)OPENSSL_VERSION_NUMBER);
 #endif
 
 	TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "Default Net Engine version: %d (%s)\n\n=====================================================\n\n", (int)turn_params.net_engine_version, turn_params.net_engine_version_txt[(int)turn_params.net_engine_version]);
@@ -1553,7 +1573,11 @@ int main(int argc, char **argv)
 	init_dynamic_ip_lists();
 
 	if (!strstr(argv[0], "turnadmin")) {
-		while (((c = getopt_long(argc, argv, OPTIONS, long_options, NULL)) != -1)) {
+
+		struct uoptions uo;
+		uo.u.m = long_options;
+
+		while (((c = getopt_long(argc, argv, OPTIONS, uo.u.o, NULL)) != -1)) {
 			switch (c){
 			case 'l':
 				set_logfile(optarg);
@@ -1564,6 +1588,9 @@ int main(int argc, char **argv)
 			case SYSLOG_OPT:
 				set_log_to_syslog(get_bool_value(optarg));
 				break;
+			case SIMPLE_LOG_OPT:
+				set_simple_log(get_bool_value(optarg));
+				break;
 			default:
 				;
 			}
@@ -1612,7 +1639,10 @@ int main(int argc, char **argv)
 
 	read_config_file(argc,argv,0);
 
-	while (((c = getopt_long(argc, argv, OPTIONS, long_options, NULL)) != -1)) {
+	struct uoptions uo;
+	uo.u.m = long_options;
+
+	while (((c = getopt_long(argc, argv, OPTIONS, uo.u.o, NULL)) != -1)) {
 		if(c != 'u')
 			set_option(c,optarg);
 	}
@@ -1621,7 +1651,7 @@ int main(int argc, char **argv)
 
 	optind = 0;
 
-	while (((c = getopt_long(argc, argv, OPTIONS, long_options, NULL)) != -1)) {
+	while (((c = getopt_long(argc, argv, OPTIONS, uo.u.o, NULL)) != -1)) {
 	  if(c == 'u') {
 	    set_option(c,optarg);
 	  }
@@ -1912,10 +1942,14 @@ static void adjust_key_file_name(char *fn, const char* file_title, int critical)
 	} else {
 
 	  full_path_to_file = find_config_file(fn, 1);
-	  FILE *f = full_path_to_file ? fopen(full_path_to_file,"r") : NULL;
-	  if(!f) {
-	    TURN_LOG_FUNC(TURN_LOG_LEVEL_WARNING,"WARNING: cannot find %s file: %s (1)\n",file_title,fn);
-	    goto keyerr;
+	  {
+		  FILE *f = full_path_to_file ? fopen(full_path_to_file,"r") : NULL;
+		  if(!f) {
+			  TURN_LOG_FUNC(TURN_LOG_LEVEL_WARNING,"WARNING: cannot find %s file: %s (1)\n",file_title,fn);
+			  goto keyerr;
+		  } else {
+			  fclose(f);
+		  }
 	  }
 
 	  if(!full_path_to_file) {
