@@ -100,7 +100,6 @@ static void close_socket_net_data(ioa_socket_handle s);
 /************** Utils **************************/
 
 static const int tcp_congestion_control = 1;
-static const int udp_congestion_control = 1;
 
 static int bufferevent_enabled(struct bufferevent *bufev, short flags)
 {
@@ -109,11 +108,13 @@ static int bufferevent_enabled(struct bufferevent *bufev, short flags)
 
 static int is_socket_writeable(ioa_socket_handle s, size_t sz, const char *msg, int option) 
 {
-  UNUSED_ARG(s);
   UNUSED_ARG(sz);
   UNUSED_ARG(msg);
   UNUSED_ARG(option);
 
+  if(!s)
+	  return 0;
+
   if(!(s->done) && !(s->broken) && !(s->tobeclosed)) {
 
     switch(s->st) {
@@ -205,6 +206,9 @@ static void log_socket_event(ioa_socket_handle s, const char *msg, int error) {
 
 int set_df_on_ioa_socket(ioa_socket_handle s, int value)
 {
+	if(!s)
+		return 0;
+
 	if(s->parent_s)
 		return 0;
 
@@ -980,33 +984,31 @@ int set_socket_options(ioa_socket_handle s)
 
 /* <<== Socket options helpers */
 
-ioa_socket_handle create_unbound_ioa_socket(ioa_engine_handle e, ioa_socket_handle parent_s, int family, SOCKET_TYPE st, SOCKET_APP_TYPE sat)
+ioa_socket_handle create_unbound_ioa_socket(ioa_engine_handle e, int family, SOCKET_TYPE st, SOCKET_APP_TYPE sat)
 {
 	evutil_socket_t fd = -1;
 	ioa_socket_handle ret = NULL;
 
-	if(!parent_s) {
-		switch (st){
-		case UDP_SOCKET:
-			fd = socket(family, SOCK_DGRAM, 0);
-			if (fd < 0) {
-				perror("UDP socket");
-				return NULL;
-			}
-			set_sock_buf_size(fd, UR_CLIENT_SOCK_BUF_SIZE);
-			break;
-		case TCP_SOCKET:
-			fd = socket(family, SOCK_STREAM, 0);
-			if (fd < 0) {
-				perror("TCP socket");
-				return NULL;
-			}
-			set_sock_buf_size(fd, UR_CLIENT_SOCK_BUF_SIZE);
-			break;
-		default:
-			/* we do not support other sockets in the relay position */
+	switch (st){
+	case UDP_SOCKET:
+		fd = socket(family, SOCK_DGRAM, 0);
+		if (fd < 0) {
+			perror("UDP socket");
 			return NULL;
 		}
+		set_sock_buf_size(fd, UR_CLIENT_SOCK_BUF_SIZE);
+		break;
+	case TCP_SOCKET:
+		fd = socket(family, SOCK_STREAM, 0);
+		if (fd < 0) {
+			perror("TCP socket");
+			return NULL;
+		}
+		set_sock_buf_size(fd, UR_CLIENT_SOCK_BUF_SIZE);
+		break;
+	default:
+		/* we do not support other sockets in the relay position */
+		return NULL;
 	}
 
 	ret = (ioa_socket*)turn_malloc(sizeof(ioa_socket));
@@ -1020,11 +1022,7 @@ ioa_socket_handle create_unbound_ioa_socket(ioa_engine_handle e, ioa_socket_hand
 	ret->sat = sat;
 	ret->e = e;
 
-	if(parent_s) {
-		add_socket_to_parent(parent_s, ret);
-	} else {
-		set_socket_options(ret);
-	}
+	set_socket_options(ret);
 
 	return ret;
 }
@@ -1095,8 +1093,6 @@ int create_relay_ioa_sockets(ioa_engine_handle e,
 		ioa_addr rtcp_local_addr;
 		addr_cpy(&rtcp_local_addr, &relay_addr);
 
-		addr_debug_print(e->verbose, &relay_addr, "Server relay addr");
-
 		int i = 0;
 		int port = 0;
 		ioa_addr local_addr;
@@ -1112,7 +1108,7 @@ int create_relay_ioa_sockets(ioa_engine_handle e,
 				if (port >= 0 && even_port > 0) {
 
 					IOA_CLOSE_SOCKET(*rtcp_s);
-					*rtcp_s = create_unbound_ioa_socket(e, NULL, relay_addr.ss.sa_family, UDP_SOCKET, RELAY_RTCP_SOCKET);
+					*rtcp_s = create_unbound_ioa_socket(e, relay_addr.ss.sa_family, UDP_SOCKET, RELAY_RTCP_SOCKET);
 					if (*rtcp_s == NULL) {
 						perror("socket");
 						IOA_CLOSE_SOCKET(*rtp_s);
@@ -1148,7 +1144,7 @@ int create_relay_ioa_sockets(ioa_engine_handle e,
 
 				IOA_CLOSE_SOCKET(*rtp_s);
 
-				*rtp_s = create_unbound_ioa_socket(e, NULL, relay_addr.ss.sa_family,
+				*rtp_s = create_unbound_ioa_socket(e, relay_addr.ss.sa_family,
 										(transport == STUN_ATTRIBUTE_TRANSPORT_TCP_VALUE) ? TCP_SOCKET : UDP_SOCKET,
 										RELAY_SOCKET);
 				if (*rtp_s == NULL) {
@@ -1318,7 +1314,7 @@ static void connect_eventcb(struct bufferevent *bev, short events, void *ptr)
 
 ioa_socket_handle ioa_create_connecting_tcp_relay_socket(ioa_socket_handle s, ioa_addr *peer_addr, connect_cb cb, void *arg)
 {
-	ioa_socket_handle ret = create_unbound_ioa_socket(s->e, NULL, s->family, s->st, TCP_RELAY_DATA_SOCKET);
+	ioa_socket_handle ret = create_unbound_ioa_socket(s->e, s->family, s->st, TCP_RELAY_DATA_SOCKET);
 
 	if(!ret) {
 		return NULL;
@@ -1493,6 +1489,9 @@ ioa_socket_handle create_ioa_socket_from_fd(ioa_engine_handle e,
 /* Only must be called for DTLS_SOCKET */
 ioa_socket_handle create_ioa_socket_from_ssl(ioa_engine_handle e, ioa_socket_handle parent_s, SSL* ssl, SOCKET_TYPE st, SOCKET_APP_TYPE sat, const ioa_addr *remote_addr, const ioa_addr *local_addr)
 {
+	if(!parent_s)
+		return NULL;
+
 	ioa_socket_handle ret = create_ioa_socket_from_fd(e, parent_s->fd, parent_s, st, sat, remote_addr, local_addr);
 
 	if(ret) {
@@ -1653,8 +1652,7 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 
 		evutil_socket_t udp_fd = -1;
 
-		if(full_detach && s->parent_s) {
-
+		if(s->parent_s && full_detach) {
 			udp_fd = socket(s->local_addr.ss.sa_family, SOCK_DGRAM, 0);
 			if (udp_fd < 0) {
 				perror("socket");
@@ -1682,6 +1680,8 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 
 		ret->magic = SOCKET_MAGIC;
 
+		ret->username_hash = s->username_hash;
+
 		ret->ssl = s->ssl;
 		ret->fd = s->fd;
 
@@ -1692,15 +1692,22 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 		ret->local_addr_known = s->local_addr_known;
 		addr_cpy(&(ret->local_addr),&(s->local_addr));
 		ret->connected = s->connected;
-		ioa_socket_handle parent_s = s->parent_s;
 		addr_cpy(&(ret->remote_addr),&(s->remote_addr));
 
-		ur_addr_map *sockets_container = s->sockets_container;
+		STRCPY(ret->orig_ctx_type, s->orig_ctx_type);
 
+		ioa_socket_handle parent_s = s->parent_s;
+		ur_addr_map *sockets_container = s->sockets_container;
+		
 		delete_socket_from_map(s);
 		delete_socket_from_parent(s);
 
-		if(full_detach && parent_s) {
+		if(udp_fd<0) {
+
+		  add_socket_to_parent(parent_s, ret);
+		  add_socket_to_map(ret,sockets_container);
+
+		} else {
 
 			ret->fd = udp_fd;
 
@@ -1722,10 +1729,6 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 			}
 
 			set_socket_options(ret);
-
-		} else {
-			add_socket_to_parent(parent_s, ret);
-			add_socket_to_map(ret,sockets_container);
 		}
 
 		ret->current_ttl = s->current_ttl;
@@ -1806,11 +1809,13 @@ void set_ioa_socket_app_type(ioa_socket_handle s, SOCKET_APP_TYPE sat) {
 
 ioa_addr* get_local_addr_from_ioa_socket(ioa_socket_handle s)
 {
-	if (s) {
+	if (s && (s->magic == SOCKET_MAGIC) && !(s->done)) {
 
 		if(s->parent_s) {
-			return get_local_addr_from_ioa_socket(s->parent_s);
-		} else if (s->local_addr_known) {
+			s = s->parent_s;
+		}
+
+		if (s->local_addr_known) {
 			return &(s->local_addr);
 		} else if (s->bound && (addr_get_port(&(s->local_addr)) > 0)) {
 			s->local_addr_known = 1;
@@ -1840,7 +1845,7 @@ ioa_addr* get_local_addr_from_ioa_socket(ioa_socket_handle s)
 
 ioa_addr* get_remote_addr_from_ioa_socket(ioa_socket_handle s)
 {
-	if (s) {
+	if (s && (s->magic == SOCKET_MAGIC) && !(s->done)) {
 
 		if (s->connected) {
 			return &(s->remote_addr);
@@ -1853,7 +1858,7 @@ int get_local_mtu_ioa_socket(ioa_socket_handle s)
 {
 	if(s) {
 		if(s->parent_s)
-			return get_local_mtu_ioa_socket(s->parent_s);
+			s = s->parent_s;
 
 		return get_socket_mtu(s->fd, s->family, (s->e && eve(s->e->verbose)));
 	}
@@ -2255,13 +2260,13 @@ static int socket_input_worker(ioa_socket_handle s)
 		addr_cpy(&remote_addr,&(s->remote_addr));
 
 	if(tcp_congestion_control && s->sub_session && s->bev) {
-	  if(s == s->sub_session->client_s) {
+	  if(s == s->sub_session->client_s && (s->sub_session->peer_s)) {
 	    if(!is_socket_writeable(s->sub_session->peer_s, STUN_BUFFER_SIZE,__FUNCTION__,0)) {
 	      if(bufferevent_enabled(s->bev,EV_READ)) {
 	    	  bufferevent_disable(s->bev,EV_READ);
 	      }
 	    }
-	  } else if(s == s->sub_session->peer_s) {
+	  } else if(s == s->sub_session->peer_s && (s->sub_session->client_s)) {
 	    if(!is_socket_writeable(s->sub_session->client_s, STUN_BUFFER_SIZE,__FUNCTION__,1)) {
 	      if(bufferevent_enabled(s->bev,EV_READ)) {
 	    	  bufferevent_disable(s->bev,EV_READ);
@@ -2306,24 +2311,23 @@ static int socket_input_worker(ioa_socket_handle s)
 				if(s->e->tls_ctx_v1_2) {
 					s->ssl = SSL_new(s->e->tls_ctx_v1_2);
 					STRCPY(s->orig_ctx_type,"TLSv1.2");
-					break;
 				}
+				break;
 #endif
 #if defined(SSL_TXT_TLSV1_1)
 			case TURN_TLS_v1_1:
 				if(s->e->tls_ctx_v1_1) {
 					s->ssl = SSL_new(s->e->tls_ctx_v1_1);
 					STRCPY(s->orig_ctx_type,"TLSv1.1");
-					break;
 				}
-
+				break;
 #endif
 			case TURN_TLS_v1_0:
 				if(s->e->tls_ctx_v1_0) {
 					s->ssl = SSL_new(s->e->tls_ctx_v1_0);
 					STRCPY(s->orig_ctx_type,"TLSv1.0");
-					break;
 				}
+				break;
 			default:
 				if(s->e->tls_ctx_ssl23) {
 					s->ssl = SSL_new(s->e->tls_ctx_ssl23);
@@ -2467,7 +2471,7 @@ static int socket_input_worker(ioa_socket_handle s)
 				nd.recv_ttl = ttl;
 				nd.recv_tos = tos;
 
-				s->read_cb(s, IOA_EV_READ, &nd, s->read_ctx);
+				s->read_cb(s, IOA_EV_READ, &nd, s->read_ctx, 1);
 
 				if(nd.nbh)
 					free_blist_elem(s->e,buf_elem);
@@ -2511,8 +2515,10 @@ static void socket_input_handler(evutil_socket_t fd, short what, void* arg)
 
 	ioa_socket_handle s = (ioa_socket_handle)arg;
 
-	if(!s)
+	if(!s) {
+		read_spare_buffer(fd);
 		return;
+	}
 
 	if((s->magic != SOCKET_MAGIC)||(s->done)) {
 		read_spare_buffer(fd);
@@ -2528,6 +2534,8 @@ static void socket_input_handler(evutil_socket_t fd, short what, void* arg)
 
 	if (!ioa_socket_tobeclosed(s))
 		socket_input_worker(s);
+	else
+		read_spare_buffer(fd);
 
 	if((s->magic != SOCKET_MAGIC)||(s->done)) {
 		TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!!%s (1) on socket, ev=%d: 0x%lx, st=%d, sat=%d\n", __FUNCTION__,(int)what,(long)s, s->st, s->sat);
@@ -2577,7 +2585,17 @@ static void socket_output_handler_bev(struct bufferevent *bev, void* arg)
 
 			ioa_socket_handle s = (ioa_socket_handle) arg;
 
-			if ((s->magic != SOCKET_MAGIC)||(s->done)||ioa_socket_tobeclosed(s)||(bev != s->bev)) {
+			if(s->in_write)
+				return;
+
+			if ((s->magic != SOCKET_MAGIC)||(s->done)||(bev != s->bev)) {
+				return;
+			}
+
+			if (s->tobeclosed) {
+				if (bufferevent_enabled(bev,EV_READ)) {
+					bufferevent_disable(bev,EV_READ);
+				}
 				return;
 			}
 
@@ -2616,27 +2634,50 @@ static void socket_output_handler_bev(struct bufferevent *bev, void* arg)
 	}
 }
 
+static int read_spare_buffer_bev(struct bufferevent *bev)
+{
+	if(bev) {
+		char some_buffer[8192];
+		bufferevent_read(bev, some_buffer, sizeof(some_buffer));
+	}
+	return 0;
+}
+
 static void socket_input_handler_bev(struct bufferevent *bev, void* arg)
 {
 
-	if (bev && arg) {
+	if (bev) {
+
+		if(!arg) {
+			read_spare_buffer_bev(bev);
+			return;
+		}
 
 		ioa_socket_handle s = (ioa_socket_handle) arg;
 
 		if(bev != s->bev) {
 			TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!! %s socket: 0x%lx: wrong bev\n", __FUNCTION__,(long)s);
+			read_spare_buffer_bev(bev);
 			return;
 		}
 
 		if((s->magic != SOCKET_MAGIC)||(s->done)) {
 			TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!!%s on socket: 0x%lx, st=%d, sat=%d\n", __FUNCTION__, (long) s, s->st, s->sat);
 			TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!! %s socket: 0x%lx was closed\n", __FUNCTION__,(long)s);
+			read_spare_buffer_bev(bev);
 			return;
 		}
 
-		while (!ioa_socket_tobeclosed(s)) {
-			if (socket_input_worker(s) <= 0)
-				break;
+		{
+			size_t cycle = 0;
+			do {
+				if(ioa_socket_tobeclosed(s)) {
+					read_spare_buffer_bev(s->bev);
+					break;
+				}
+				if (socket_input_worker(s) <= 0)
+					break;
+			} while((cycle++<128) && (s->bev));
 		}
 
 		if((s->magic != SOCKET_MAGIC)||(s->done)) {
@@ -2645,29 +2686,7 @@ static void socket_input_handler_bev(struct bufferevent *bev, void* arg)
 			return;
 		}
 
-		if (ioa_socket_tobeclosed(s)) {
-			switch(s->sat) {
-			case TCP_CLIENT_DATA_SOCKET:
-			case TCP_RELAY_DATA_SOCKET:
-			{
-				tcp_connection *tc = s->sub_session;
-				if(tc) {
-					delete_tcp_connection(tc);
-				}
-			}
-			break;
-			default:
-			{
-				ts_ur_super_session *ss = s->session;
-				if (ss) {
-					turn_turnserver *server = (turn_turnserver *)ss->server;
-					if (server) {
-						shutdown_client_connection(server, ss, 0, "TCP socket buffer operation error (input handler)");
-					}
-				}
-			}
-			}
-		}
+		close_ioa_socket_after_processing_if_necessary(s);
 	}
 }
 
@@ -2991,8 +3010,10 @@ int send_data_from_ioa_socket_nbh(ioa_socket_handle s, ioa_addr* dest_addr,
 {
 	int ret = -1;
 
-	if(!s)
+	if(!s) {
+		ioa_network_buffer_delete(NULL, nbh);
 		return -1;
+	}
 
 	if (s->done || (s->fd == -1)) {
 		TURN_LOG_FUNC(
@@ -3028,21 +3049,25 @@ int send_data_from_ioa_socket_nbh(ioa_socket_handle s, ioa_addr* dest_addr,
 
 							ret = (int) ioa_network_buffer_get_size(nbh);
 
-							if(!udp_congestion_control || is_socket_writeable(s,(size_t)ret,__FUNCTION__,2)) {
-							  if (bufferevent_write(
-										s->bev,
+							if (!tcp_congestion_control || is_socket_writeable(
+									s, (size_t) ret, __FUNCTION__, 2)) {
+								s->in_write = 1;
+								if (bufferevent_write(s->bev,
 										ioa_network_buffer_data(nbh),
-										ioa_network_buffer_get_size(nbh))
-							      < 0) {
-							    ret = -1;
-							    perror("bufev send");
-							    log_socket_event(s, "socket write failed, to be closed",1);
-							    s->tobeclosed = 1;
-							    s->broken = 1;
-							  }
+										ioa_network_buffer_get_size(nbh)) < 0) {
+									ret = -1;
+									perror("bufev send");
+									log_socket_event(
+											s,
+											"socket write failed, to be closed",
+											1);
+									s->tobeclosed = 1;
+									s->broken = 1;
+								}
+								s->in_write = 0;
 							} else {
-							  //drop the packet
-							  ;
+								//drop the packet
+								;
 							}
 						}
 					} else if (s->ssl) {
@@ -3217,21 +3242,23 @@ int ioa_socket_tobeclosed(ioa_socket_handle s)
 		}
 
 		if(s->done) {
-			TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!! %s: ceck on already closed socket: 0x%lx, st=%d, sat=%d\n",__FUNCTION__,(long)s,s->st,s->sat);
+			TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!! %s: check on already closed socket: 0x%lx, st=%d, sat=%d\n",__FUNCTION__,(long)s,s->st,s->sat);
 			TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "!!! %s socket: 0x%lx was closed\n", __FUNCTION__,(long)s);
 			return 1;
 		}
-		if(s->broken) {
+		if(s->tobeclosed) {
+			return 1;
+		} else if(s->broken) {
+			s->tobeclosed = 1;
 			log_socket_event(s, "socket broken", 0);
 			return 1;
-		} else if(s->tobeclosed) {
-			log_socket_event(s, "socket to be closed", 0);
-			return 1;
 		} else if(s->fd < 0) {
+			s->tobeclosed = 1;
 			log_socket_event(s, "socket fd<0", 0);
 			return 1;
 		} else if(s->ssl) {
 			if(SSL_get_shutdown(s->ssl)) {
+				s->tobeclosed = 1;
 				log_socket_event(s, "socket SSL shutdown", 0);
 				return 1;
 			}
@@ -3246,6 +3273,32 @@ void set_ioa_socket_tobeclosed(ioa_socket_handle s)
 		s->tobeclosed = 1;
 }
 
+static u32bits string_hash(const u08bits *str) {
+
+  u32bits hash = 0;
+  int c = 0;
+
+  while ((c = *str++))
+    hash = c + (hash << 6) + (hash << 16) - hash;
+
+  return hash;
+}
+
+int check_username_hash(ioa_socket_handle s, u08bits *username)
+{
+	if(s && username && username[0]) {
+		return (s->username_hash == string_hash(username));
+	}
+	return 1;
+}
+
+void set_username_hash(ioa_socket_handle s, u08bits *username)
+{
+	if(s && username && username[0]) {
+		s->username_hash = string_hash(username);
+	}
+}
+
 /*
  * Network buffer functions
  */
@@ -3350,10 +3403,17 @@ void turn_report_allocation_set(void *a, turn_time_t lifetime, int refresh)
 			turn_turnserver *server = (turn_turnserver*)ss->server;
 			if(server) {
 				ioa_engine_handle e = turn_server_get_engine(server);
-				if(e && e->verbose) {
+				if(e && e->verbose && ss->client_session.s) {
 					if(ss->client_session.s->ssl) {
-						TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO,"session %018llu: %s, username=<%s>, lifetime=%lu, cipher=%s, method=%s (%s)\n", (unsigned long long)ss->id, status, (char*)ss->username, (unsigned long)lifetime, SSL_get_cipher(ss->client_session.s->ssl),
-							turn_get_ssl_method(ss->client_session.s->ssl, ss->client_session.s->orig_ctx_type),ss->client_session.s->orig_ctx_type);
+					  TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO,
+							"session %018llu: %s, username=<%s>, lifetime=%lu, cipher=%s, method=%s (%s)\n", 
+							(unsigned long long)ss->id, 
+							status, 
+							(char*)ss->username, 
+							(unsigned long)lifetime, 
+							SSL_get_cipher(ss->client_session.s->ssl),
+							turn_get_ssl_method(ss->client_session.s->ssl, ss->client_session.s->orig_ctx_type),
+							ss->client_session.s->orig_ctx_type);
 					} else {
 						TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO,"session %018llu: %s, username=<%s>, lifetime=%lu\n", (unsigned long long)ss->id, status, (char*)ss->username, (unsigned long)lifetime);
 					}
@@ -3534,7 +3594,6 @@ void* allocate_super_memory_region_func(super_memory_t *r, size_t size, const ch
 		size_t i = 0;
 		char *region = NULL;
 		size_t *rsz = NULL;
-		size_t chn = 0;
 		for(i=0;i<=r->sm_chunk;++i) {
 
 			size_t left = (size_t)r->sm_total_sz - r->sm_allocated[i];
@@ -3544,7 +3603,6 @@ void* allocate_super_memory_region_func(super_memory_t *r, size_t size, const ch
 			} else {
 				region = r->super_memory[i];
 				rsz = r->sm_allocated + i;
-				chn = i;
 				break;
 			}
 		}
@@ -3558,13 +3616,9 @@ void* allocate_super_memory_region_func(super_memory_t *r, size_t size, const ch
 			r->sm_allocated[r->sm_chunk] = 0;
 			region = r->super_memory[r->sm_chunk];
 			rsz = r->sm_allocated + r->sm_chunk;
-			chn = r->sm_chunk;
 		}
 
 		{
-			if(chn)
-				TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO,"(%s:%s:%d): allocated super memory: region id = %u, chunk=%lu, chunks=%lu, total=%lu, allocated=%lu, want=%lu\n",file,func,line,(unsigned int)r->id, (unsigned long)chn, (unsigned long)(r->sm_chunk+1), (unsigned long)r->sm_total_sz, (unsigned long)r->sm_allocated[chn],(unsigned long)size);
-
 			char* ptr = region + *rsz;
 
 			ns_bzero(ptr, size);
