@@ -91,6 +91,12 @@ struct cb_socket_message {
 	stun_tid tid;
 	ioa_socket_handle s;
 	int message_integrity;
+	ioa_net_data nd;
+	int can_resume;
+};
+
+struct cancelled_session_message {
+	turnsession_id id;
 };
 
 struct relay_server {
@@ -112,6 +118,7 @@ struct message_to_relay {
 	union {
 		struct socket_message sm;
 		struct cb_socket_message cb_sm;
+		struct cancelled_session_message csm;
 	} m;
 };
 
@@ -173,6 +180,7 @@ struct _ioa_socket
 	SOCKET_TYPE st;
 	SOCKET_APP_TYPE sat;
 	SSL* ssl;
+	int in_write;
 	char orig_ctx_type[16];
 	int bound;
 	int local_addr_known;
@@ -205,6 +213,8 @@ struct _ioa_socket
 	struct bufferevent *conn_bev;
 	connect_cb conn_cb;
 	void *conn_arg;
+	//Transferable sockets user data
+	u32bits username_hash;
 	//Accept:
 	struct evconnlistener *list_ev;
 	accept_cb acb;
@@ -265,6 +275,8 @@ int set_raw_socket_tos_options(evutil_socket_t fd, int family);
 int set_socket_options_fd(evutil_socket_t fd, int tcp, int family);
 int set_socket_options(ioa_socket_handle s);
 
+int send_session_cancellation_to_relay(turnsession_id sid);
+
 ///////////////////////// SUPER MEMORY ////////
 
 #define allocate_super_memory_engine(e,size) allocate_super_memory_engine_func(e, size, __FILE__, __FUNCTION__, __LINE__)
