@@ -121,11 +121,15 @@ static const char *CLI_HELP_STR[] =
    "",
    "  pu [udp|tcp|dtls|tls]- print current users",
    "",
+   "  lr - log reset",
+   "",
    "  aas ip[:port} - add an alternate server reference",
    "  das ip[:port] - delete an alternate server reference",
    "  atas ip[:port] - add a TLS alternate server reference",
    "  dtas ip[:port] - delete a TLS alternate server reference",
    "",
+   "  cs <session-id> - cancel session, forcefully"
+   "",
    NULL};
 
 static const char *CLI_GREETING_STR[] = {
@@ -193,6 +197,14 @@ static void myprintf(struct cli_session *cs, const char *format, ...)
 	}
 }
 
+static void log_reset(struct cli_session* cs)
+{
+	if(cs) {
+	  reset_rtpprintf();
+	  myprintf(cs,"  log reset done\n");
+	}
+}
+
 static void print_str_array(struct cli_session* cs, const char** sa)
 {
   if(cs && sa) {
@@ -429,26 +441,26 @@ static int print_session(ur_map_key_type key, ur_map_value_type value, void *arg
 		struct turn_session_info *tsi = (struct turn_session_info *)value;
 
 		if(csarg->users) {
+			const char *pn=csarg->pname;
+			if(pn[0]) {
+				if(!strcmp(pn,"TLS") || !strcmp(pn,"tls") || !strcmp(pn,"Tls")) {
+					if(tsi->client_protocol != TLS_SOCKET)
+						return 0;
+				} else if(!strcmp(pn,"DTLS") || !strcmp(pn,"dtls") || !strcmp(pn,"Dtls")) {
+					if(tsi->client_protocol != DTLS_SOCKET)
+						return 0;
+				} else if(!strcmp(pn,"TCP") || !strcmp(pn,"tcp") || !strcmp(pn,"Tcp")) {
+					if(tsi->client_protocol != TCP_SOCKET)
+						return 0;
+				} else if(!strcmp(pn,"UDP") || !strcmp(pn,"udp") || !strcmp(pn,"Udp")) {
+					if(tsi->client_protocol != UDP_SOCKET)
+						return 0;
+				} else {
+					return 0;
+				}
+			}
 			ur_string_map_value_type value;
 			if(!ur_string_map_get(csarg->users, (ur_string_map_key_type)(char*)tsi->username, &value)) {
-				const char *pn=csarg->pname;
-				if(pn[0]) {
-					if(!strcmp(pn,"TLS") || !strcmp(pn,"tls") || !strcmp(pn,"Tls")) {
-						if(tsi->client_protocol != TLS_SOCKET)
-							return 0;
-					} else if(!strcmp(pn,"DTLS") || !strcmp(pn,"dtls") || !strcmp(pn,"Dtls")) {
-						if(tsi->client_protocol != DTLS_SOCKET)
-							return 0;
-					} else if(!strcmp(pn,"TCP") || !strcmp(pn,"tcp") || !strcmp(pn,"Tcp")) {
-						if(tsi->client_protocol != TCP_SOCKET)
-							return 0;
-					} else if(!strcmp(pn,"UDP") || !strcmp(pn,"udp") || !strcmp(pn,"Udp")) {
-						if(tsi->client_protocol != UDP_SOCKET)
-							return 0;
-					} else {
-						return 0;
-					}
-				}
 				value = (ur_string_map_value_type)csarg->users_number;
 				csarg->users_number += 1;
 				csarg->user_counters = (size_t*)turn_realloc(csarg->user_counters,
@@ -534,6 +546,14 @@ static int print_session(ur_map_key_type key, ur_map_value_type value, void *arg
 	return 0;
 }
 
+static void cancel_session(struct cli_session* cs, const char* ssid)
+{
+	if(cs && cs->ts && ssid && *ssid) {
+		turnsession_id sid = strtoull(ssid,NULL,10);
+		send_session_cancellation_to_relay(sid);
+	}
+}
+
 static void print_sessions(struct cli_session* cs, const char* pn, int exact_match, int print_users)
 {
 	if(cs && cs->ts && pn) {
@@ -963,6 +983,12 @@ static int run_cli_input(struct cli_session* cs, const char *buf0, unsigned int
 			} else if(strstr(cmd,"ps") == cmd) {
 				print_sessions(cs,cmd+2,1,0);
 				type_cli_cursor(cs);
+			} else if(strstr(cmd,"cs ") == cmd) {
+				cancel_session(cs,cmd+3);
+				type_cli_cursor(cs);
+			} else if(strstr(cmd,"lr") == cmd) {
+				log_reset(cs);
+				type_cli_cursor(cs);
 			} else if(strstr(cmd,"cc ") == cmd) {
 				change_cli_param(cs,cmd+3);
 				type_cli_cursor(cs);
@@ -1084,7 +1110,7 @@ static void cliserver_input_handler(struct evconnlistener *l, evutil_socket_t fd
 
 	clisession->bev = bufferevent_socket_new(cliserver.event_base,
 					fd,
-					BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+					BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
 	bufferevent_setcb(clisession->bev, cli_socket_input_handler_bev, NULL,
 			cli_eventcb_bev, clisession);
 	bufferevent_setwatermark(clisession->bev, EV_READ|EV_WRITE, 0, BUFFEREVENT_HIGH_WATERMARK);
