@@ -589,19 +589,45 @@ static redisContext *get_redis_connection(void)
 				redisconnection = redisConnect(ip, port);
 			}
 
-			if (!redisconnection) {
-				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "Cannot initialize Redis DB connection\n");
-			} else {
-				if (co->password) {
-					turnFreeRedisReply(redisCommand(redisconnection, "AUTH %s", co->password));
-				}
-				if (co->dbname) {
-					turnFreeRedisReply(redisCommand(redisconnection, "select %s", co->dbname));
-				}
-				if (!donot_print_connection_success) {
-					TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "Redis DB sync connection success: %s\n", turn_params.users_params.userdb);
+			if (redisconnection) {
+				if(redisconnection->err) {
+					if(redisconnection->errstr[0]) {
+						TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "Redis: %s\n",redisconnection->errstr);
+					}
+					redisFree(redisconnection);
+					redisconnection = NULL;
+				} else if (co->password) {
+					void *reply = redisCommand(redisconnection, "AUTH %s", co->password);
+					if(!reply) {
+						if(redisconnection->err && redisconnection->errstr[0]) {
+							TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "Redis: %s\n",redisconnection->errstr);
+						}
+						redisFree(redisconnection);
+						redisconnection = NULL;
+					} else {
+						turnFreeRedisReply(reply);
+						if (co->dbname) {
+							reply = redisCommand(redisconnection, "select %s", co->dbname);
+							if(!reply) {
+								if(redisconnection->err && redisconnection->errstr[0]) {
+									TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "Redis: %s\n",redisconnection->errstr);
+								}
+								redisFree(redisconnection);
+								redisconnection = NULL;
+							} else {
+								turnFreeRedisReply(reply);
+							}
+						}
+					}
 				}
 			}
+
+			if (!redisconnection) {
+				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "Cannot initialize Redis DB connection\n");
+			} else if (!donot_print_connection_success) {
+				TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "Redis DB sync connection success: %s\n", turn_params.users_params.userdb);
+			}
+
 			RyconninfoFree(co);
 		}
 	}
@@ -1174,7 +1200,7 @@ int check_new_allocation_quota(u08bits *user)
 		ur_string_map_lock(turn_params.users_params.users.alloc_counters);
 		if (turn_params.users_params.users.total_quota && (turn_params.users_params.users.total_current_allocs >= turn_params.users_params.users.total_quota)) {
 			ret = -1;
-		} else {
+		} else if(username[0]){
 			ur_string_map_value_type value = 0;
 			if (!ur_string_map_get(turn_params.users_params.users.alloc_counters, (ur_string_map_key_type) username, &value)) {
 				value = (ur_string_map_value_type) 1;
@@ -1189,6 +1215,8 @@ int check_new_allocation_quota(u08bits *user)
 					++(turn_params.users_params.users.total_current_allocs);
 				}
 			}
+		} else {
+			++(turn_params.users_params.users.total_current_allocs);
 		}
 		turn_free(username,strlen(username)+1);
 		ur_string_map_unlock(turn_params.users_params.users.alloc_counters);
@@ -1423,7 +1451,7 @@ static int list_users(int is_st)
 			redisReply *reply = NULL;
 
 			if(!is_st) {
-				redisReply *reply = (redisReply*)redisCommand(rc, "keys turn/user/*/key");
+				reply = (redisReply*)redisCommand(rc, "keys turn/user/*/key");
 				if(reply) {
 
 					if (reply->type == REDIS_REPLY_ERROR)
