@@ -81,6 +81,7 @@ int mandatory_channel_padding = 0;
 int negative_test = 0;
 int negative_protocol_test = 0;
 int dos = 0;
+int random_disconnect = 0;
 
 SHATYPE shatype = SHATYPE_SHA1;
 
@@ -119,8 +120,9 @@ static char Usage[] =
   "		Without this option, by default, SHA1 is used.\n"
   "	-M	ICE Mobility engaged.\n"
   "	-I	Do not set permissions on TURN relay endpoints\n"
-  "     	(for testing the non-standard server relay functionality).\n"
+  "		(for testing the non-standard server relay functionality).\n"
   "	-G	Generate extra requests (create permissions, channel bind).\n"
+  " -B  Random disconnect after a few initial packets.\n"
   "Options:\n"
   "	-l	Message length (Default: 100 Bytes).\n"
   "	-i	Certificate file (for secure connections only, optional).\n"
@@ -139,7 +141,8 @@ static char Usage[] =
   "	-u	STUN/TURN user name.\n"
   "	-w	STUN/TURN user password.\n"
   "	-W	TURN REST API authentication secret. Is not compatible with -A option.\n"
-  "	-C	TURN REST API username/timestamp separator symbol (character). The default value is ':'.\n";
+  "	-C	TURN REST API timestamp/username separator symbol (character). The default value is ':'.\n"
+  "	-F	<cipher-suite> Cipher suite for TLS/DTLS. Default value is DEFAULT.\n";
 
 //////////////////////////////////////////////////
 
@@ -192,8 +195,11 @@ int main(int argc, char **argv)
 
 	ns_bzero(local_addr, sizeof(local_addr));
 
-	while ((c = getopt(argc, argv, "d:p:l:n:L:m:e:r:u:w:i:k:z:W:C:E:F:vsyhcxXgtTSAPDNOUHMRIG")) != -1) {
+	while ((c = getopt(argc, argv, "d:p:l:n:L:m:e:r:u:w:i:k:z:W:C:E:F:vsyhcxXgtTSAPDNOUHMRIGB")) != -1) {
 		switch (c){
+		case 'B':
+			random_disconnect = 1;
+			break;
 		case 'G':
 			extra_requests = 1;
 			break;
@@ -350,7 +356,7 @@ int main(int argc, char **argv)
 		}
 		{
 			char new_uname[1025];
-			const unsigned long exp_time = 600; /* one day */
+			const unsigned long exp_time = 3600 * 24; /* one day */
 			if(g_uname[0]) {
 			  snprintf(new_uname,sizeof(new_uname),"%lu%c%s",(unsigned long)time(NULL) + exp_time,rest_api_separator, (char*)g_uname);
 			} else {
