@@ -31,9 +31,9 @@
 #ifndef __IOADEFS__
 #define __IOADEFS__
 
-#define TURN_SERVER_VERSION "3.2.3.1"
+#define TURN_SERVER_VERSION "3.2.4.1"
 #define TURN_SERVER_VERSION_NAME "Marshal West"
-#define TURN_SOFTWARE "Citrix-"TURN_SERVER_VERSION" '"TURN_SERVER_VERSION_NAME"'"
+#define TURN_SOFTWARE "Citrix-" TURN_SERVER_VERSION " '" TURN_SERVER_VERSION_NAME "'"
 
 #if (defined(__unix__) || defined(unix)) && !defined(USG)
 #include <sys/param.h>
@@ -143,14 +143,15 @@ typedef u32bits turn_time_t;
 #endif
 
 #define STRCPY(dst,src) \
-	do { \
+	do { if((const char*)(dst) != (const char*)(src)) { \
 		if(sizeof(dst)==sizeof(char*))\
 			strcpy(((char*)(dst)),(const char*)(src));\
 		else {\
-			strncpy((char*)(dst),(const char*)(src),sizeof((dst)));\
-			((char*)(dst))[sizeof((dst))-1] = 0; \
+			size_t szdst = sizeof((dst));\
+			strncpy((char*)(dst),(const char*)(src),szdst);\
+			((char*)(dst))[szdst-1] = 0;\
 		}\
-	} while(0)
+	} } while(0)
 
 ////////////////// Security ////////////////////////////
 
@@ -170,6 +171,20 @@ typedef enum _SHATYPE SHATYPE;
 
 #define SHA_TOO_WEAK (426)
 
+//////////////// KERNEL-LEVEL CHANNEL HANDLERS /////////
+
+#if !defined(TURN_CHANNEL_HANDLER_KERNEL)
+#define TURN_CHANNEL_HANDLER_KERNEL void*
+#endif
+
+#if !defined(CREATE_TURN_CHANNEL_KERNEL)
+#define CREATE_TURN_CHANNEL_KERNEL(channel_number, address_family_client, address_family_peer, protocol_client, client_addr, local_addr, local_relay_addr, peer_addr) ((TURN_CHANNEL_HANDLER_KERNEL)(1))
+#endif
+
+#if !defined(DELETE_TURN_CHANNEL_KERNEL)
+#define DELETE_TURN_CHANNEL_KERNEL(handler)
+#endif
+
 ////////////////////////////////////////////////////////
 
 #ifdef __cplusplus
