@@ -62,7 +62,6 @@ void clear_allocation(allocation *a)
 				if(tc) {
 					delete_tcp_connection(tc);
 					a->tcs.elems[i] = NULL;
-					break;
 				}
 			}
 			turn_free(a->tcs.elems,sz*sizeof(tcp_connection*));
@@ -215,10 +214,14 @@ static turn_permission_info* get_from_turn_permission_hashtable(turn_permission_
 }
 
 static void ch_info_clean(ch_info* c) {
-  if(c) {
-    IOA_EVENT_DEL(c->lifetime_ev);
-    ns_bzero(c,sizeof(ch_info));
-  }
+  if (c) {
+		if (c->kernel_channel) {
+			DELETE_TURN_CHANNEL_KERNEL(c->kernel_channel);
+			c->kernel_channel = 0;
+		}
+		IOA_EVENT_DEL(c->lifetime_ev);
+		ns_bzero(c,sizeof(ch_info));
+	}
 }
 
 static int delete_channel_info_from_allocation_map(ur_map_key_type key, ur_map_value_type value)
@@ -598,6 +601,17 @@ tcp_connection *get_and_clean_tcp_connection_by_id(ur_map *map, tcp_connection_i
 	return NULL;
 }
 
+tcp_connection *get_tcp_connection_by_id(ur_map *map, tcp_connection_id id)
+{
+	if(map) {
+		ur_map_value_type t = 0;
+		if (ur_map_get(map, (ur_map_key_type)id, &t) && t) {
+			return (tcp_connection*)t;
+		}
+	}
+	return NULL;
+}
+
 tcp_connection *get_tcp_connection_by_peer(allocation *a, ioa_addr *peer_addr)
 {
 	if(a && peer_addr) {
